/* :mode=JP-SQL:folding=explicit:collapseFolds=1:indentSize=3: */

DROP TABLE TREATMENT_FILES_CONTENTS IF EXISTS;
DROP TABLE TREATMENT_DATA_CONTENTS IF EXISTS;
DROP TABLE TREATMENT_FILES IF EXISTS;
DROP TABLE TREATMENT_DATA IF EXISTS;
DROP TABLE TREATMENT IF EXISTS;
DROP TABLE PATIENT_FILES_CONTENTS IF EXISTS;
DROP TABLE PATIENT_FILES IF EXISTS;
DROP TABLE PATIENT IF EXISTS;
DROP TABLE USER IF EXISTS;
DROP TABLE GPD_DBINFO IF EXISTS;

* if (*dropping == true) * break

CREATE MEMORY TABLE GPD_DBINFO
   (
   DB_VERSION INTEGER NOT NULL,
   ADMIN_PASSWORD_HASH VARCHAR(64) DEFAULT ''
   );

CREATE CACHED TABLE USER
   (
   username VARCHAR(40) PRIMARY KEY,
   passwordHash VARCHAR(64)
   );

CREATE CACHED TABLE PATIENT
   (
   patient_id INTEGER GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY,
   username VARCHAR(40) NOT NULL,
   name VARCHAR(100) DEFAULT '<name not given>',
   info VARCHAR(32768) DEFAULT '',
   last_treatment_time TIMESTAMP(0) WITHOUT TIME ZONE,
   finalized BOOLEAN DEFAULT FALSE,
   CONSTRAINT patient_owner_username FOREIGN KEY (username) REFERENCES USER (username)
   );

CREATE CACHED TABLE PATIENT_FILES
   (
   patient_id INTEGER NOT NULL,
   filename VARCHAR(256),
   CONSTRAINT patient_file_id FOREIGN KEY (patient_id) REFERENCES PATIENT (patient_id)
   	ON DELETE CASCADE
   );

CREATE CACHED TABLE PATIENT_FILES_CONTENTS
   (
   patient_id INTEGER NOT NULL,
   filename VARCHAR(256),
   contents BLOB,
   checksum INTEGER,
   CONSTRAINT patient_files_contents_ref FOREIGN KEY (patient_id) REFERENCES PATIENT (patient_id)
   );

CREATE CACHED TABLE TREATMENT
   (
   treatment_id INTEGER GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY,
   parent_patient_id INTEGER NOT NULL,
   info VARCHAR(32768),
   treatment_time TIMESTAMP(0) WITHOUT TIME ZONE,
   finalized BOOLEAN DEFAULT FALSE,
   CONSTRAINT treatment_patient_id FOREIGN KEY (parent_patient_id) REFERENCES PATIENT (patient_id)
   );

CREATE CACHED TABLE TREATMENT_DATA
   (
   treatment_id INTEGER NOT NULL,
   datastring VARCHAR(256),
   CONSTRAINT treatment_data_id FOREIGN KEY (treatment_id) REFERENCES TREATMENT (treatment_id)
   	ON DELETE CASCADE
   );

CREATE CACHED TABLE TREATMENT_FILES
   (
   treatment_id INTEGER NOT NULL,
   filename VARCHAR(256),
   CONSTRAINT treatment_file_id FOREIGN KEY (treatment_id) REFERENCES TREATMENT (treatment_id)
   	ON DELETE CASCADE
   );

CREATE CACHED TABLE TREATMENT_DATA_CONTENTS
   (
   treatment_id INTEGER NOT NULL,
   datastring VARCHAR(256),
   contents BLOB,
   checksum INTEGER,
   CONSTRAINT treatment_data_contents_ref FOREIGN KEY (treatment_id) REFERENCES TREATMENT (treatment_id)
   );

CREATE CACHED TABLE TREATMENT_FILES_CONTENTS
   (
   treatment_id INTEGER NOT NULL,
   filename VARCHAR(256),
   contents BLOB,
   checksum INTEGER,
   CONSTRAINT treatment_files_contents_ref FOREIGN KEY (treatment_id) REFERENCES TREATMENT (treatment_id)
   );
