/* :mode=JP-SQL:folding=explicit:collapseFolds=1:indentSize=3:noTabs=true: */

DROP FUNCTION ROUTINE_NAMES IF EXISTS;

DROP PROCEDURE STORE_PATIENT_FILE_CONTENTS IF EXISTS;
DROP PROCEDURE STORE_TREATMENT_DATA_CONTENTS IF EXISTS;
DROP PROCEDURE STORE_TREATMENT_FILE_CONTENTS IF EXISTS;
DROP PROCEDURE GET_ALL_TREATMENTS IF EXISTS;
DROP PROCEDURE GET_TREATMENT IF EXISTS;
DROP PROCEDURE GET_TREATMENTS IF EXISTS;
DROP PROCEDURE DELETE_USER_PATIENTS IF EXISTS;
DROP PROCEDURE DELETE_PATIENT_TREATMENTS IF EXISTS;
DROP PROCEDURE DELETE_TREATMENT IF EXISTS;
DROP PROCEDURE GET_PATIENTS IF EXISTS;
DROP PROCEDURE GET_PATIENT IF EXISTS;


* if (*dropping == true) * break

-- the '\.' and '.;' demarcations around each procedure definition below are 
-- to put SqlTool into raw mode, so that special characters are passed verbatim.


-- This function relies on ANSI SQL standard tables, which HSQLDB supports but others
-- may not. It is not necessary for GPD's operation, and so can be omitted from any
-- translations.

\.
CREATE FUNCTION ROUTINE_NAMES () RETURNS TABLE(name VARCHAR(128))
   LANGUAGE SQL SPECIFIC routine_names_function READS SQL DATA
 BEGIN ATOMIC
   RETURN TABLE (SELECT ROUTINE_NAME FROM INFORMATION_SCHEMA.ROUTINES);
 END
.;

-- ================================================================================

\.
CREATE PROCEDURE GET_PATIENT (IN pid INTEGER)
    LANGUAGE SQL SPECIFIC get_patient_procedure READS SQL DATA DYNAMIC RESULT SETS 2
 BEGIN ATOMIC
   DECLARE patients CURSOR WITH RETURN FOR
	 SELECT patient_id, username, name, info, last_treatment_time, finalized
	    FROM PATIENT WHERE patient_id = pid
      FOR READ ONLY;
   DECLARE files CURSOR WITH RETURN FOR
	 SELECT patient_id AS parent_patient_id, filename FROM PATIENT_FILES WHERE patient_id = pid
      FOR READ ONLY;
   OPEN patients;
   OPEN files;
 END
.;

\.
CREATE PROCEDURE GET_PATIENTS (IN username_ VARCHAR(40))
    LANGUAGE SQL SPECIFIC get_patients_procedure READS SQL DATA DYNAMIC RESULT SETS 2
 BEGIN ATOMIC
   DECLARE patients CURSOR WITH RETURN FOR
	 SELECT patient_id, username, name, info, last_treatment_time, finalized
	    FROM PATIENT WHERE username = username_ ORDER BY patient_id
      FOR READ ONLY;
   DECLARE files CURSOR WITH RETURN FOR 
	 SELECT pf.patient_id AS parent_patient_id, pf.filename FROM
	    PATIENT_FILES pf INNER JOIN PATIENT p 
	    ON pf.patient_id = p.patient_id AND p.username = username_
      FOR READ ONLY;
   OPEN patients;
   OPEN files;
 END
.;

\.
CREATE PROCEDURE DELETE_TREATMENT (IN treatmentId_ INTEGER)
      LANGUAGE SQL SPECIFIC delete_treatment_procedure MODIFIES SQL DATA
 BEGIN ATOMIC
   DELETE FROM TREATMENT_DATA_CONTENTS WHERE treatment_id = treatmentId_;
   DELETE FROM TREATMENT_DATA WHERE treatment_id = treatmentId_;
   DELETE FROM TREATMENT_FILES_CONTENTS WHERE treatment_id = treatmentId_;
   DELETE FROM TREATMENT_FILES WHERE treatment_id = treatmentId_;
   DELETE FROM TREATMENT WHERE treatment_id = treatmentId_;
 END
.;

-- If deletePatient is TRUE, then the patient will be deleted from the PATIENT table;
-- otherwise, only the patient's treatments will be deleted.
\.
CREATE PROCEDURE DELETE_PATIENT_TREATMENTS (IN patientId_ INTEGER, IN deletePatient BOOLEAN)
      LANGUAGE SQL SPECIFIC delete_patient_procedure MODIFIES SQL DATA
 BEGIN ATOMIC
   FOR SELECT treatment_id FROM TREATMENT WHERE parent_patient_id = patientId_ DO
      CALL DELETE_TREATMENT(treatment_id);
   END FOR;
   IF deletePatient = TRUE THEN
      DELETE FROM PATIENT_FILES_CONTENTS WHERE patient_id = patientId_;
      DELETE FROM PATIENT_FILES WHERE patient_id = patientId_;
      DELETE FROM PATIENT WHERE patient_id = patientId_;
   END IF;
 END
.;

-- If deleteUser is TRUE, then the user will be deleted from the USER table;
-- otherwise, only the user's patients will be deleted.
\.
CREATE PROCEDURE DELETE_USER_PATIENTS (IN username_ VARCHAR(40), IN deleteUser BOOLEAN)
      LANGUAGE SQL SPECIFIC delete_user_patients_procedure MODIFIES SQL DATA
 BEGIN ATOMIC
   FOR SELECT patient_id FROM PATIENT WHERE username = username_ DO
      CALL DELETE_PATIENT_TREATMENTS(patient_id, TRUE);
   END FOR;
   IF deleteUser = TRUE THEN
      DELETE FROM USER WHERE username = username_;
   END IF;
 END
.;

\.
CREATE PROCEDURE GET_TREATMENT (IN treatmentId_ INTEGER)
    LANGUAGE SQL SPECIFIC get_treatment_procedure READS SQL DATA DYNAMIC RESULT SETS 3
 BEGIN ATOMIC
   DECLARE treatments CURSOR WITH RETURN FOR
	 SELECT treatment_id, parent_patient_id, info, treatment_time, finalized
	    FROM TREATMENT WHERE treatment_id = treatmentId_
      FOR READ ONLY;
   DECLARE datastrings CURSOR WITH RETURN FOR
	 SELECT treatment_id AS parent_treatment_id, datastring 
	 	FROM TREATMENT_DATA WHERE treatment_id = treatmentId_
      FOR READ ONLY;
   DECLARE files CURSOR WITH RETURN FOR
	 SELECT treatment_id AS parent_treatment_id, filename 
	 	FROM TREATMENT_FILES WHERE treatment_id = treatmentId_
      FOR READ ONLY;
   OPEN treatments;
   OPEN datastrings;
   OPEN files;
 END
.;

\.
CREATE PROCEDURE GET_TREATMENTS (IN patientId_ INTEGER)
    LANGUAGE SQL SPECIFIC get_treatments_procedure READS SQL DATA DYNAMIC RESULT SETS 3
 BEGIN ATOMIC
   DECLARE treatments CURSOR WITH RETURN FOR
	 SELECT treatment_id, parent_patient_id, info, treatment_time, finalized
	    FROM TREATMENT WHERE parent_patient_id = patientId_ ORDER BY treatment_id DESC
      FOR READ ONLY;
   DECLARE datastrings CURSOR WITH RETURN FOR
	 SELECT td.treatment_id AS parent_treatment_id, td.datastring 
	 	FROM TREATMENT_DATA td INNER JOIN TREATMENT t
	 	  ON td.treatment_id = t.treatment_id AND t.parent_patient_id = patientId_
      FOR READ ONLY;
   DECLARE files CURSOR WITH RETURN FOR
	 SELECT tf.treatment_id AS parent_treatment_id, tf.filename 
	 	FROM TREATMENT_FILES tf INNER JOIN TREATMENT t
	 	  ON tf.treatment_id = t.treatment_id AND t.parent_patient_id = patientId_
      FOR READ ONLY;
   OPEN treatments;
   OPEN datastrings;
   OPEN files;
 END
.;

\.
CREATE PROCEDURE GET_ALL_TREATMENTS ()
    LANGUAGE SQL SPECIFIC get_all_treatments_procedure READS SQL DATA DYNAMIC RESULT SETS 3
 BEGIN ATOMIC
   DECLARE treatments CURSOR WITH RETURN FOR 
      SELECT treatment_id, parent_patient_id, info, treatment_time, finalized FROM TREATMENT FOR READ ONLY;
   DECLARE datastrings CURSOR WITH RETURN FOR 
      SELECT treatment_id AS parent_treatment_id, datastring FROM TREATMENT_DATA FOR READ ONLY;
   DECLARE files CURSOR WITH RETURN FOR 
      SELECT treatment_id AS parent_treatment_id, filename FROM TREATMENT_FILES FOR READ ONLY;
   OPEN treatments;
   OPEN datastrings;
   OPEN files;
 END
.;

-- These three below file-content procedures are not actually used by the default HSQLDB mapper,
-- but serve as a reference to different ways the functionality could be implemented, for DBs
-- with varying levels of ANSI SQL support.

\.
CREATE PROCEDURE STORE_TREATMENT_FILE_CONTENTS (IN treatment_id_ INTEGER, IN filename_ VARCHAR(256), 
                                                IN contents_ BLOB, IN checksum_ INTEGER)
   LANGUAGE SQL SPECIFIC store_treatment_file_contents_procedure MODIFIES SQL DATA
 BEGIN ATOMIC
   DECLARE N INTEGER;
   SET N = (SELECT COUNT(*) FROM TREATMENT_FILES_CONTENTS WHERE treatment_id = treatment_id_ AND filename = filename_);
   CASE N
      WHEN 0 THEN INSERT INTO TREATMENT_FILES_CONTENTS VALUES (treatment_id_, filename_, contents_, checksum_);
      ELSE UPDATE TREATMENT_FILES_CONTENTS SET contents = contents_, checksum = checksum_
                WHERE treatment_id = treatment_id_ AND filename = filename_; 
   END CASE;
 END
.;

\.
CREATE PROCEDURE STORE_TREATMENT_DATA_CONTENTS (IN treatment_id_ INTEGER, IN datastring_ VARCHAR(256), 
                                                IN contents_ BLOB, IN checksum_ INTEGER)
   LANGUAGE SQL SPECIFIC store_treatment_data_contents_procedure MODIFIES SQL DATA
 BEGIN ATOMIC
   DECLARE N INTEGER;
   UPDATE TREATMENT_DATA_CONTENTS SET contents = contents_, checksum = checksum_
      WHERE treatment_id = treatment_id_ AND datastring = datastring_;
   GET DIAGNOSTICS N = ROW_COUNT;
   IF N = 0 THEN
      INSERT INTO TREATMENT_DATA_CONTENTS (treatment_id, datastring, contents, checksum)
        VALUES (treatment_id_, datastring_, contents_, checksum_);
   END IF;
 END
.;

\.
CREATE PROCEDURE STORE_PATIENT_FILE_CONTENTS (IN patient_id_ INTEGER, IN filename_ VARCHAR(256), 
                                              IN contents_ BLOB, IN checksum_ INTEGER)
   LANGUAGE SQL SPECIFIC store_patient_file_contents_procedure MODIFIES SQL DATA
 BEGIN ATOMIC
    MERGE INTO PATIENT_FILES_CONTENTS AS PFC USING 
        (VALUES(patient_id_, filename_, contents_, checksum_)) AS vals(patient_id, filename, contents, checksum)
      ON PFC.patient_id = vals.patient_id AND PFC.filename = vals.filename
      WHEN MATCHED THEN UPDATE SET PFC.contents = vals.contents, PFC.checksum = vals.checksum
      WHEN NOT MATCHED THEN INSERT VALUES (vals.patient_id, vals.filename, vals.contents, vals.checksum);
 END
.;
