/* :mode=transact-sql:folding=explicit:collapseFolds=1:indentSize=3: */

DROP TABLE IF EXISTS TREATMENT_FILES_CONTENTS;
DROP TABLE IF EXISTS TREATMENT_DATA_CONTENTS;
DROP TABLE IF EXISTS TREATMENT_FILES;
DROP TABLE IF EXISTS TREATMENT_DATA;
DROP TABLE IF EXISTS TREATMENT;
DROP TABLE IF EXISTS PATIENT_FILES_CONTENTS;
DROP TABLE IF EXISTS PATIENT_FILES;
DROP TABLE IF EXISTS PATIENT;
DROP TABLE IF EXISTS [USER];
DROP TABLE IF EXISTS GPD_DBINFO;

* if (*dropping == true) * break

CREATE TABLE GPD_DBINFO
   (
   DB_VERSION INT NOT NULL,
   ADMIN_PASSWORD_HASH NVARCHAR(64) DEFAULT N''
   );

CREATE TABLE [USER]
   (
   username NVARCHAR(40) PRIMARY KEY,
   passwordHash NVARCHAR(64)
   );

CREATE TABLE PATIENT
   (
   patient_id INT IDENTITY (1,1) NOT NULL PRIMARY KEY,
   username NVARCHAR(40) NOT NULL,
   name NVARCHAR(100) DEFAULT N'<name not given>',
   info NVARCHAR(max) DEFAULT N'',
   last_treatment_time DATETIME2(0),
   finalized BIT NOT NULL DEFAULT 0,
   CONSTRAINT patient_owner_username FOREIGN KEY (username) REFERENCES [USER] (username)
   );

CREATE TABLE PATIENT_FILES
   (
   patient_id INT NOT NULL,
   filename NVARCHAR(256),
   CONSTRAINT patient_file_id FOREIGN KEY (patient_id) REFERENCES PATIENT (patient_id)
   	ON DELETE CASCADE
   );

CREATE TABLE PATIENT_FILES_CONTENTS
   (
   patient_id INT NOT NULL,
   filename NVARCHAR(256),
   contents VARBINARY(max),
   checksum INT,
   CONSTRAINT patient_files_contents_ref FOREIGN KEY (patient_id) REFERENCES PATIENT (patient_id)
   );

CREATE TABLE TREATMENT
   (
   treatment_id INT IDENTITY (1,1) NOT NULL PRIMARY KEY,
   parent_patient_id INT NOT NULL,
   info NVARCHAR(max),
   treatment_time DATETIME2(0),
   finalized BIT NOT NULL DEFAULT 0,
   CONSTRAINT treatment_patient_id FOREIGN KEY (parent_patient_id) REFERENCES PATIENT (patient_id)
   );

CREATE TABLE TREATMENT_DATA
   (
   treatment_id INT NOT NULL,
   datastring NVARCHAR(256),
   CONSTRAINT treatment_data_id FOREIGN KEY (treatment_id) REFERENCES TREATMENT (treatment_id)
   	ON DELETE CASCADE
   );

CREATE TABLE TREATMENT_FILES
   (
   treatment_id INT NOT NULL,
   filename NVARCHAR(256),
   CONSTRAINT treatment_file_id FOREIGN KEY (treatment_id) REFERENCES TREATMENT (treatment_id)
   	ON DELETE CASCADE
   );

CREATE TABLE TREATMENT_DATA_CONTENTS
   (
   treatment_id INT NOT NULL,
   datastring NVARCHAR(256),
   contents VARBINARY(max),
   checksum INT,
   CONSTRAINT treatment_data_contents_ref FOREIGN KEY (treatment_id) REFERENCES TREATMENT (treatment_id)
   );

CREATE TABLE TREATMENT_FILES_CONTENTS
   (
   treatment_id INT NOT NULL,
   filename NVARCHAR(256),
   contents VARBINARY(max),
   checksum INT,
   CONSTRAINT treatment_files_contents_ref FOREIGN KEY (treatment_id) REFERENCES TREATMENT (treatment_id)
   );
