/* :mode=transact-sql:folding=explicit:collapseFolds=1:indentSize=3: */

DROP PROCEDURE IF EXISTS GET_ALL_TREATMENTS;
DROP PROCEDURE IF EXISTS GET_TREATMENT;
DROP PROCEDURE IF EXISTS GET_TREATMENTS;
DROP PROCEDURE IF EXISTS DELETE_USER_PATIENTS;
DROP PROCEDURE IF EXISTS DELETE_PATIENT_TREATMENTS;
DROP PROCEDURE IF EXISTS DELETE_TREATMENT;
DROP PROCEDURE IF EXISTS GET_PATIENTS;
DROP PROCEDURE IF EXISTS GET_PATIENT;


* if (*dropping == true) * break

-- the '\.' and '.;' demarcations around each procedure definition below are 
-- to put SqlTool into raw mode, so that special characters are passed verbatim.


-- ================================================================================

-- For examples, see
-- https://docs.microsoft.com/en-us/sql/t-sql/statements/create-procedure-transact-sql?view=sql-server-ver15#Examples

\.
CREATE PROCEDURE GET_PATIENT
   @pid INT
AS
BEGIN
   SELECT patient_id, username, name, info, last_treatment_time, finalized
	    FROM PATIENT WHERE patient_id = @pid;
   SELECT patient_id AS parent_patient_id, filename FROM PATIENT_FILES WHERE patient_id = @pid;
END
.;

\.
CREATE PROCEDURE GET_PATIENTS
   @username_ NVARCHAR(40)
AS
BEGIN
   SELECT patient_id, username, name, info, last_treatment_time, finalized
	    FROM PATIENT WHERE username = @username_ ORDER BY patient_id;
   SELECT pf.patient_id AS parent_patient_id, pf.filename FROM
	    PATIENT_FILES pf INNER JOIN PATIENT p 
	    ON pf.patient_id = p.patient_id AND p.username = @username_;
END
.;

\.
CREATE PROCEDURE DELETE_TREATMENT
   @treatmentId_ INT
AS
BEGIN
   DELETE FROM TREATMENT_DATA_CONTENTS WHERE treatment_id = @treatmentId_;
   DELETE FROM TREATMENT_DATA WHERE treatment_id = @treatmentId_;
   DELETE FROM TREATMENT_FILES_CONTENTS WHERE treatment_id = @treatmentId_;
   DELETE FROM TREATMENT_FILES WHERE treatment_id = @treatmentId_;
   DELETE FROM TREATMENT WHERE treatment_id = @treatmentId_;
END
.;

-- If deletePatient is TRUE, then the patient will be deleted from the PATIENT table;
-- otherwise, only the patient's treatments will be deleted.

-- Take a look at https://stackoverflow.com/questions/11852782/t-sql-loop-over-query-results
-- and https://docs.microsoft.com/en-us/sql/t-sql/language-elements/fetch-transact-sql?view=sql-server-ver15#examples
\.
CREATE PROCEDURE DELETE_PATIENT_TREATMENTS
   @patientId_ INT, @deletePatient BIT
AS
BEGIN
   DECLARE @treatmentId INT;
   DECLARE tid_cursor CURSOR FOR
      SELECT treatment_id FROM TREATMENT WHERE parent_patient_id = @patientId_;
   OPEN tid_cursor;
   FETCH NEXT FROM tid_cursor INTO @treatmentId;
   WHILE @@FETCH_STATUS = 0
   BEGIN
      EXEC DELETE_TREATMENT @treatmentId;
      FETCH NEXT FROM tid_cursor INTO @treatmentId;
   END
   CLOSE tid_cursor;
   DEALLOCATE tid_cursor;
   
   IF @deletePatient = 1
   BEGIN
      DELETE FROM PATIENT_FILES_CONTENTS WHERE patient_id = @patientId_;
      DELETE FROM PATIENT_FILES WHERE patient_id = @patientId_;
      DELETE FROM PATIENT WHERE patient_id = @patientId_;
   END
END
.;

-- If deleteUser is TRUE, then the user will be deleted from the USER table;
-- otherwise, only the user's patients will be deleted.
\.
CREATE PROCEDURE DELETE_USER_PATIENTS
   @username_ NVARCHAR(40), @deleteUser BIT
AS
BEGIN
   DECLARE @patientId INT;
   DECLARE pid_cursor CURSOR FOR
      SELECT patient_id FROM PATIENT WHERE username = @username_;
   OPEN pid_cursor;
   FETCH NEXT FROM pid_cursor INTO @patientId;
   WHILE @@FETCH_STATUS = 0
   BEGIN
      EXEC DELETE_PATIENT_TREATMENTS @patientId, 1;
      FETCH NEXT FROM pid_cursor INTO @patientId;
   END
   CLOSE pid_cursor;
   DEALLOCATE pid_cursor;
   
   IF @deleteUser = 1
   BEGIN
      DELETE FROM [USER] WHERE username = @username_;
   END
END
.;

\.
CREATE PROCEDURE GET_TREATMENT
   @treatmentId_ INT
AS
BEGIN
   SELECT treatment_id, parent_patient_id, info, treatment_time, finalized
      FROM TREATMENT WHERE treatment_id = @treatmentId_;
   SELECT treatment_id AS parent_treatment_id, datastring 
      FROM TREATMENT_DATA WHERE treatment_id = @treatmentId_;
   SELECT treatment_id AS parent_treatment_id, filename 
      FROM TREATMENT_FILES WHERE treatment_id = @treatmentId_;
END

.;

\.
CREATE PROCEDURE GET_TREATMENTS
   @patientId_ INT
AS
BEGIN
   SELECT treatment_id, parent_patient_id, info, treatment_time, finalized
      FROM TREATMENT WHERE parent_patient_id = @patientId_ ORDER BY treatment_id DESC;
   SELECT td.treatment_id AS parent_treatment_id, td.datastring 
      FROM TREATMENT_DATA td INNER JOIN TREATMENT t
        ON td.treatment_id = t.treatment_id AND t.parent_patient_id = @patientId_;
   SELECT tf.treatment_id AS parent_treatment_id, tf.filename 
      FROM TREATMENT_FILES tf INNER JOIN TREATMENT t
        ON tf.treatment_id = t.treatment_id AND t.parent_patient_id = @patientId_;
END
.;

\.
CREATE PROCEDURE GET_ALL_TREATMENTS
AS
BEGIN
   SELECT treatment_id, parent_patient_id, info, treatment_time, finalized FROM TREATMENT;
   SELECT treatment_id AS parent_treatment_id, datastring FROM TREATMENT_DATA;
   SELECT treatment_id AS parent_treatment_id, filename FROM TREATMENT_FILES;
END
.;
