<#ftl output_format="plainText">

<#-- The name of the generated random ID field -->
<#assign genet_ID_field = "GENET ID ANONYM">

<#-- Function to generate a random patient ID. Right now it takes the last 8 characters
     of a INT hash of the patient ID. -->
<#function genet_ID item parent_item>
  <#local f1 = item["LAST NAME"]!parent_item["LAST NAME"]!"N/A">
  <#local f2 = item["FIRST NAME"]!parent_item["FIRST NAME"]!"N/A">
  <#local f3 = item["DATE OF BIRTH (DD.MM.YYYY)"]!parent_item["DATE OF BIRTH (DD.MM.YYYY)"]!"N/A">
  <#return utils.intHash(f1+f2+f3)>
</#function>

<#-- 
  Output macro
  Parameters
    fields - sequence of field names to consider for output
    items - the actual data items (patient or treatment) with values keyed by fields
    parent_item - if the item we're examining does not contain a particular field,
                  we'll attempt to find it in this hash.
    output_header - true to output a CSV header row
    output_items - true to output the values in 'items'
    strip_units - true to remove units from item values (using utils.stripUnits)
    filterfields - a sequence of field names to exclude from output
-->
<#macro output_csv_info fields items parent_item={} 
                        output_header=true output_items=true strip_units=true
                        filterfields=[]>
<#if output_header>
  <#list fields as field>
    <#if !filterfields?seq_contains(field)>
      ${utils.escapeCsv(field)}<#sep>,</#sep><#t>
    </#if>
  </#list>

</#if>
<#if output_items>
  <#list items as item>
    <#list fields as field>
      <#if !filterfields?seq_contains(field)>
        <#if field == genet_ID_field>
          <#local val = genet_ID(item, parent_item)>
        <#else>
          <#local val = item[field]!parent_item[field]!"">
          <#if val?length != 0>
            <#if strip_units><#local val = utils.stripUnit(val)></#if>
            <#list units_to_strip as unit>
                <#if val?ends_with(unit)>
                  <#local val = val?remove_ending(unit)>
                  <#break>
                </#if>
            </#list>
            <#local val = utils.escapeCsv(val)>
          </#if>
        </#if>
        ${val}<#sep>,</#sep><#t>
      </#if>
    </#list>
  
  </#list>
</#if>
</#macro>
<#-- End output macro -->

<#--assign patient_field_names = patientFieldNames?sequence?sort-->
<#--assign treatment_field_names = treatmentFieldNames?sequence?sort-->

<#assign patient_field_names = ["GENET ID", "PATIENT ID", "PATIENT COUNTER", "LAST NAME", "FIRST NAME", "DATE OF BIRTH (DD.MM.YYYY)", "AGE", "SEX", "DIAGNOSIS (ICD-10)", "DIAGNOSIS (ICD-11)", 
	"DIAGNOSIS (DSM-5)", "FIRST COURSE EVER OF ECT", "YEAR OF FIRST ECT"]>
<#assign treatment_field_names = ["PATIENT STATUS", "WEIGHT", "THYMATRON SERIAL NUMBER", "DATE OF TREATMENT", "TREATMENT DATE", "PATIENT TREATMENT NUMBER", "ECT COURSE", "RESTIMULATION", 
	"ELECTRODE POSITION", "ANESTHETIC", "ANESTHETIC DOSE (MG)", "ANESTHETIC ADD", "ANESTHETIC ADD DOSE (MG)", "RELAXANT", 
	"RELAXANT DOSE (MG)", "RELAXANT ADD", "RELAXANT ADD DOSE (MG)", "ANALGESICS", "ANALGESICS DOSE (MG)", "ANALGESICS ADD", 
	"ANALGESICS ADD DOSE (MG)", "ANTIEMETICS", "ANTIEMETICS DOSE (MG)", "ANTIEMETICS ADD", "ANTIEMETICS ADD DOSE (MG)", 
	"ANTIHYPERTENSIVES", "ANTIHYPERTENSIVE DOSE (MG)", "ANTICHOLINERGIC", "ANTICHOLINERGIC DOSE (MG)", "CONCURRENT MEDICATION_1", "CONCURRENT MEDICATION_1 DOSE (MG)", 
	"CONCURRENT MEDICATION_2", "CONCURRENT MEDICATION_2 DOSE (MG)", "CONCURRENT MEDICATION_3", "CONCURRENT MEDICATION_3 DOSE (MG)", "CGI (1-NORMAL, 7-AMONG THE MOST EXTREMLY ILL PATIENTS)", 
	"PROGRAM SELECTED", "% ENERGY SET", "CHARGE DELIVERED", "CURRENT", "STIMULUS DURATION", "FREQUENCY", "PULSE WIDTH", "STATIC IMPEDANCE", 
	"DYNAMIC IMPEDANCE", "EEG ENDPOINT", "EMG ENDPOINT", "BASE HEART RATE", "PEAK HEART RATE", "AVERAGE SEIZURE ENERGY INDEX", "POSTICTAL SUPPRESSION INDEX", 
	"MAXIMUM SUSTAINED POWER", "TIME TO PEAK POWER", "MAXIMUM SUSTAINED COHERENCE", "TIME TO PEAK COHERENCE", "EARLY ICTAL AMPLITUDE", "MIDICTAL AMPLITUDE", 
	"POST-ICTAL AMPLITUDE", "ANESTHETICS ADMINISTERED MARKER #1", "RELAXANT ADMINISTERED MARKER #1", "POST-TREATMENT START MARKER #1", "SEIZURE END MARKER #1", 
	"EEG SEIZURE LENGTH", "MOTOR SEIZURE LENGTH", "MEDICATION_CHANGES 1", "MEDICATION_CHANGES 1 DOSE (MG)","MEDICATION_CHANGES 2", "MEDICATION_CHANGES 2 DOSE (MG)","MEDICATION_CHANGES 3", 
	"MEDICATION_CHANGES 3 DOSE (MG)", "SEIZURE EVALUATION (1-INSUFFICIENT, 2-SUFFICIENT, 3-IDEAL)", "RECOMMENDATION"]>

<#-- Note the spaces in each of the unit text entries below! -->
<#assign units_to_strip = [" %", " mC", " A", " Sec", " Hz", " mSec", " Ohm", " b/m", " uV", " uV^2", " µV²"," s"]>

<#assign timestamp = utils.timestamp()>
<#--assign genet_ID = genet_ID?left_pad(20, "0")-->

<#-- Write info for all treatments in all patients -->
<#assign filename="Anonymized-Genet-Treatment-Summary-${timestamp}.csv">
<@outputTo file="${filename}">
<#-- First we write the header row -->
<@output_csv_info fields=(patient_field_names + treatment_field_names) strip_units=false
                  filterfields=["PATIENT ID", "PATIENT COUNTER", "LAST NAME", "FIRST NAME", 
	"DATE OF BIRTH (DD.MM.YYYY)",  "YEAR OF FIRST ECT", "DATE OF TREATMENT", "FIRST COURSE EVER OF ECT"]
                  items=[] output_items=false/>
<#-- and then go through all the patients' treatments writing the data columns. Note
     that if a value for a data column (i.e. a field) is not found in the treatment,
     we search the patient's fields. -->
<#list patients as patient>
  <@output_csv_info fields=patient_field_names + treatment_field_names filterfields=["PATIENT ID", "PATIENT COUNTER", "LAST NAME", "FIRST NAME", 
	"DATE OF BIRTH (DD.MM.YYYY)",  "YEAR OF FIRST ECT", "DATE OF TREATMENT", "FIRST COURSE EVER OF ECT"]
                    items=patient.treatments  parent_item=patient
                    output_header=false strip_units=false/>
</#list>
</@outputTo>